@@ -3,13 +3,14 @@ import gc
 import time
 import shelve
 import numpy as np
-from ase import Atoms
+from ase import Atoms, units
 from copy import deepcopy
 from functools import partial
 from torch.utils.data import Dataset
 from monty.serialization import loadfn
 from multiprocessing import Pool, cpu_count
 from collections.abc import MutableSequence
+from .base_potential import ZBL
 
 
 class Database():#MutableSequence):
@@ -67,6 +68,12 @@ class Database():#MutableSequence):
     def store(self, structure_file, function, storage, ase_db=None):
         """ Map structures to descriptors and store them, including features, to database.
         If compute is False, print pre-computed descriptors message. """
+        if function['base_potential']:
+            self.base_potential = ZBL(function['base_potential']['inner'],
+                                      function['base_potential']['outer'])
+        else:
+            self.base_potential = None
+
         if storage:
             if os.path.isdir(structure_file) or structure_file.find('json') > 0:
                 fmt = 'json'
@@ -219,13 +226,22 @@ class Database():#MutableSequence):
             d['rdxdr'] = rdxdr.reshape([N, L, 9])[:, :, [0, 4, 8, 1, 2, 5]]
             #d['rdxdr'] = np.einsum('ijklm->iklm', d['rdxdr'])\
             #.reshape([shp[0], shp[2], shp[3]*shp[4]])[:, :, [0, 4, 8, 1, 2, 5]]  #need to change
-
-        d['energy'] = np.asarray(data['energy'])
-        d['force'] = np.asarray(data['force'])
-        if data['stress'] is not None:
-            d['stress'] = np.asarray(data['stress'])
+        
+        #print(len(data['structure']))
+        if self.base_potential:
+            base_d = self.base_potential.calculate(data['structure'])
         else:
-            d['stress'] = data['stress']
+            base_d = {'energy': 0., 'force': 0., 'stress': 0.}
+        
+        #print(data['energy'])
+        #print(base_d['energy'])
+
+        d['energy'] = np.asarray(data['energy'] - base_d['energy'])
+        d['force'] = np.asarray(data['force']) - base_d['force']
+        if data['stress'] is not None:
+            d['stress'] = np.asarray(data['stress']) - base_d['stress'] / units.GPa
+        else:
+            d['stress'] = data['stress'] 
         d['group'] = data['group']
 
         return d
@@ -269,7 +285,7 @@ def compute_descriptor(function, structure):
 
     elif function['type'] in ['EAD', 'ead']:
             from pyxtal_ff.descriptors.EAD import EAD
-            d = EAMD(function['parameters'],
+            d = EAD(function['parameters'],
                      function['Rc'],
                      True, True,
                      function['cutoff']).calculate(structure)
@@ -289,10 +305,6 @@ def compute_descriptor(function, structure):
         raise NotImplementedError(msg)
     
     if d['rdxdr'] is not None:
-        #shp = d['rdxdr'].shape
-        #d['rdxdr'] = np.einsum('ijklm->iklm', d['rdxdr'])\
-        #    .reshape([shp[0], shp[2], shp[3]*shp[4]])[:, :, [0,4,8,1,2,5]]
-
         N = d['x'].shape[0]
         L = d['x'].shape[1]
         rdxdr = np.zeros([N, L, 3, 3])
@@ -355,19 +367,18 @@ def parse_json(path, N=None, Random=False):
             # vasp default output: XX YY ZZ XY YZ ZX
             # pyxtal_ff/lammps use: XX YY ZZ XY XZ YZ
             # Here we assume the sequence is lammps
-            if d['group'] == 'Elastic' and 'Mo 3x3x3 cell' in d['description']: #this is very likely a wrong dataset
+            if d['group'] == 'Elastic' and 'Mo 3x3x3 cell' in d['description']:
                 stress = None
                 group = 'NoElastic'
             elif 'virial_stress' in d[key]: #kB to GPa
-                s = [-1*s/10 for s in d[key]['virial_stress']] 
-
-                if d['group'] == 'Ni3Mo': #to fix the issue
+                s = [s/10 for s in d[key]['virial_stress']] 
+                if d['group'] == 'Ni3Mo' or d['element'] == 'Cu': #just tentative fix
                     stress = [s[0], s[1], s[2], s[3], s[4], s[5]]
                 else:
                     stress = [s[0], s[1], s[2], s[3], s[5], s[4]]
 
             elif 'stress' in d[key]: #kB to GPa
-                s = [-1*s/10 for s in d[key]['stress']]
+                s = [s/10 for s in d[key]['stress']]
                 if d['group'] == 'Ni3Mo': #to fix the issue
                     stress = [s[0], s[1], s[2], s[3], s[4], s[5]]
                 else:
@@ -589,7 +600,7 @@ def parse_ase_db(db_path, N=None, Random=False):
         for i, row in enumerate(db.select()):
             structure = db.get_atoms(row.id)
             if "stress" in row.data:
-                stress = row.stress
+                stress = row.data["stress"]
             else:
                 stress = None
 
@@ -598,9 +609,18 @@ def parse_ase_db(db_path, N=None, Random=False):
             else:
                 group = None
 
+            if "dft_energy" in row.data:
+                eng = row.data["dft_energy"]
+            else:
+                eng = row.data["energy"]
+
+            if "dft_force" in row.data:
+                force = row.data["dft_force"]
+            else:
+                force = row.data["force"]
             data.append({'structure': structure,
-                         'energy': row.data["dft_energy"], 
-                         'force': row.data["dft_force"],
+                         'energy': eng,
+                         'force': force,
                          'group': group,
                          'stress': stress})
 
